/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2016 OX Software GmbH
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/text/format/pagestyles', [
    'io.ox/office/tk/utils',
    'io.ox/office/editframework/model/stylecollection'
], function (Utils, StyleCollection) {

    'use strict';

    var // definitions for page attributes
        DEFINITIONS = {

            /**
             * Total width of a single page, in 1/100 of millimeters.
             */
            width: { def: 21000 },

            /**
             * Total height of a single page, in 1/100 of millimeters.
             */
            height: { def: 29700 },

            /**
             * the actual paper format (portrait or landscape)
             */
            orientation: { def: 'portrait' },

            /**
             * Margin between left page border and editing area, in 1/100 of
             * millimeters.
             */
            marginLeft: { def: 1500 },

            /**
             * Margin between right page border and editing area, in 1/100 of
             * millimeters.
             */
            marginRight: { def: 1500 },

            /**
             * Margin between top page border and editing area, in 1/100 of
             * millimeters.
             */
            marginTop: { def: 1500 },

            /**
             * Margin between bottom page border and editing area, in 1/100 of
             * millimeters.
             */
            marginBottom: { def: 1500 },

            /**
             * Margin between top page border and top header border, in 1/100 of millimeters
             */
            marginHeader: { def: 0 },

            /**
             * Margin between bottom page border and bottom footer border, in 1/100 of millimeters
             */
            marginFooter: { def: 0 },

            /**
             * Whether to use separate header/footer settings for the first page.
             *
             */
            firstPage: { def: false },

            /**
             * Whether to use separate header/footer settings for even and odd pages, and mirror margin settings accordingly.
             */
            evenOddPages: { def: false }

        };

    // private global functions ===============================================

    /**
     * Will be called for every page whose attributes have been changed.
     *
     * @param {jQuery} page
     *  The page container element whose character attributes have been
     *  changed, as jQuery object.
     *
     * @param {Object} mergedAttributes
     *  A map of attribute maps (name/value pairs), keyed by attribute family,
     *  containing the effective attribute values merged from style sheets and
     *  explicit attributes.
     */
    function updatePageFormatting(page, mergedAttributes) {

        var // the page attributes of the passed attribute map
            pageAttributes = mergedAttributes.page,
            // effective page width (at least 2cm)
            pageWidth = Math.max(pageAttributes.width, 2000),
            // effective page height (at least 2cm)
            pageHeight = Math.max(pageAttributes.height, 2000),
            // left page margin
            leftMargin = Math.max(pageAttributes.marginLeft, 0),
            // right page margin
            rightMargin = Math.max(pageAttributes.marginRight, 0),
            // total horizontal margin
            horizontalMargin = leftMargin + rightMargin,
            // top page margin
            topMargin = Math.max(pageAttributes.marginTop, 0),
            // bottom page margin
            bottomMargin = Math.max(pageAttributes.marginBottom, 0),
            // total vertical margin
            verticalMargin = topMargin + bottomMargin;

        // restrict left/right margin to keep an editing area of at least 1cm
        if ((horizontalMargin > 0) && (pageWidth - horizontalMargin < 1000)) {
            // Change margins according to ratio of original left/right margins
            // (e.g. keep left margin twice as big as right margin, if
            // specified in the original attribute values).
            leftMargin = (pageWidth - 1000) * (leftMargin / horizontalMargin);
            rightMargin = (pageWidth - 1000) * (rightMargin / horizontalMargin);
            horizontalMargin = leftMargin + rightMargin;
        }

        // restrict top/bottom margin to keep an editing area of at least 1cm
        if ((verticalMargin > 0) && (pageHeight - verticalMargin < 1000)) {
            // Change margins according to ratio of original top/bottom margins
            // (e.g. keep top margin twice as big as bottom margin, if
            // specified in the original attribute values).
            topMargin = (pageHeight - 1000) * (topMargin / verticalMargin);
            bottomMargin = (pageHeight - 1000) * (bottomMargin / verticalMargin);
            verticalMargin = topMargin + bottomMargin;
        }

        // Set CSS attributes. Page is in 'content-size' mode to be able to
        // override padding on small devices without altering the size of the
        // editing area, thus width/height needs to be set without padding.
        page.css({
            width: Utils.convertHmmToCssLength(pageWidth - horizontalMargin, 'mm', 0.1),
            // TODO: change to 'height' when page layout is supported
            minHeight: Utils.convertHmmToCssLength(pageHeight - verticalMargin, 'mm', 0.1),
            paddingLeft: Utils.convertHmmToCssLength(leftMargin, 'mm', 0.1),
            paddingRight: Utils.convertHmmToCssLength(rightMargin, 'mm', 0.1),
            paddingTop: Utils.convertHmmToCssLength(topMargin, 'mm', 0.1),
            paddingBottom: Utils.convertHmmToCssLength(bottomMargin, 'mm', 0.1)
        });
    }

    // class PageStyles =======================================================

    /**
     * Contains the style sheets for page formatting attributes. The CSS
     * formatting will be read from and written to the page container elements.
     *
     * @constructor
     *
     * @extends StyleCollection
     *
     * @param {TextModel} docModel
     *  The text document model containing instance.
     */
    function PageStyles(docModel) {

        // base constructor ---------------------------------------------------

        StyleCollection.call(this, docModel, 'page', { formatHandler: updatePageFormatting });

        // initialization -----------------------------------------------------

        // register the attribute definitions for the style family
        docModel.registerAttributeDefinitions('page', DEFINITIONS);

        // destroy all class members on destruction
        this.registerDestructor(function () {
            docModel = null;
        });

    } // class PageStyles

    // exports ================================================================

    // derive this class from class StyleCollection
    return StyleCollection.extend({ constructor: PageStyles });

});
